@@ -3,7 +3,7 @@
     <name>AddressBookPage</name>
     <message>
         <source>Right-click to edit address or label</source>
-        <translation type="unfinished">ঠিকানা বা লেবেল পরিবর্তন করতে ডান ক্লিক করুন।</translation>
+        <translation type="unfinished">ঠিকানা বা লেবেল সম্পাদনা করতে ডান-ক্লিক করুন</translation>
     </message>
     <message>
         <source>Create a new address</source>
@@ -19,7 +19,7 @@
     </message>
     <message>
         <source>&amp;Copy</source>
-        <translation type="unfinished">&amp;কপি</translation>
+        <translation type="unfinished">এবং কপি করুন</translation>
     </message>
     <message>
         <source>C&amp;lose</source>
@@ -33,6 +33,10 @@
         <source>Enter address or label to search</source>
         <translation type="unfinished">খুঁজতে ঠিকানা বা লেবেল লিখুন</translation>
     </message>
+    <message>
+        <source>Export the data in the current tab to a file</source>
+        <translation type="unfinished">বর্তমান ট্যাবের তথ্যগুলো একটি আলাদা নথিতে লিপিবদ্ধ করুন </translation>
+    </message>
     <message>
         <source>&amp;Delete</source>
         <translation type="unfinished">&amp;মুছুন</translation>
@@ -41,6 +45,18 @@
         <source>Choose the address to send coins to</source>
         <translation type="unfinished">কয়েন পাঠানোর ঠিকানা বাছাই করুন</translation>
     </message>
+    <message>
+        <source>Choose the address to receive coins with</source>
+        <translation type="unfinished">কয়েন গ্রহণ করার ঠিকানা বাছাই করুন।</translation>
+    </message>
+    <message>
+        <source>Sending addresses</source>
+        <translation type="unfinished">ঠিকানাগুলো পাঠানো হচ্ছে।</translation>
+    </message>
+    <message>
+        <source>Receiving addresses</source>
+        <translation type="unfinished">ঠিকানাগুলো গ্রহণ করা হচ্ছে।</translation>
+    </message>
     <message>
         <source>These are your Bitcoin addresses for receiving payments. Use the 'Create new receiving address' button in the receive tab to create new addresses.
 Signing is only possible with addresses of the type 'legacy'.</source>
@@ -52,8 +68,23 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <translation type="unfinished">কমা দিয়ে আলাদা করা ফাইল</translation>
     </message>
     </context>
+<context>
+    <name>AddressTableModel</name>
+    <message>
+        <source>Label</source>
+        <translation type="unfinished">টিকেট</translation>
+    </message>
+    <message>
+        <source>Address</source>
+        <translation type="unfinished">ঠিকানা</translation>
+    </message>
+    </context>
 <context>
     <name>BitcoinApplication</name>
+    <message>
+        <source>Settings file %1 might be corrupt or invalid.</source>
+        <translation type="unfinished">1%1 সেটিংস ফাইল টি সম্ভবত নষ্ট বা করাপ্ট</translation>
+    </message>
     <message>
         <source>Runaway exception</source>
         <translation type="unfinished">পলাতক ব্যতিক্রম</translation>
@@ -73,6 +104,16 @@ Signing is only possible with addresses of the type 'legacy'.</source>
 </context>
 <context>
     <name>QObject</name>
+    <message>
+        <source>Do you want to reset settings to default values, or to abort without making changes?</source>
+        <extracomment>Explanatory text shown on startup when the settings file cannot be read. Prompts user to make a choice between resetting or aborting.</extracomment>
+        <translation type="unfinished">আপনি কি ডিফল্ট মানগুলিতে সেটিংস রিসেট করতে চান, নাকি পরিবর্তন না করেই বাতিল করতে চান?</translation>
+    </message>
+    <message>
+        <source>A fatal error occurred. Check that settings file is writable, or try running with -nosettings.</source>
+        <extracomment>Explanatory text shown on startup when the settings file could not be written. Prompts user to check that we have the ability to write to the file. Explains that the user has the option of running without a settings file.</extracomment>
+        <translation type="unfinished">একটি জটিল ত্রুটি হয়েছে। সেটিং ফাইল টি রাইটেবল কিনা চেক করুন, অথবা -nosettings দিয়ে রান করার চেষ্টা করুন</translation>
+    </message>
     <message>
         <source>%1 didn't yet exit safely…</source>
         <translation type="unfinished">%1 এখনো নিরাপদে বের হয়নি</translation>
@@ -130,43 +171,16 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         </translation>
     </message>
     </context>
-<context>
-    <name>bitcoin-core</name>
-    <message>
-        <source>SQLiteDatabase: Unexpected application id. Expected %u, got %u</source>
-        <translation type="unfinished">এস. কিয়ু. লাইট ডাটাবেস : অপ্রত্যাশিত এপ্লিকেশন আই.ডি. প্রত্যাশিত %u, পাওয়া গেলো %u </translation>
-    </message>
-    <message>
-        <source>Starting network threads…</source>
-        <translation type="unfinished">নেটওয়ার্ক থ্রেড শুরু হচ্ছে...</translation>
-    </message>
-    <message>
-        <source>The specified config file %s does not exist</source>
-        <translation type="unfinished">নির্দিষ্ট কনফিগ ফাইল %s এর অস্তিত্ব নেই</translation>
-    </message>
-    <message>
-        <source>Unable to open %s for writing</source>
-        <translation type="unfinished">লেখার জন্যে %s খোলা যাচ্ছে না</translation>
-    </message>
-    <message>
-        <source>Unknown new rules activated (versionbit %i)</source>
-        <translation type="unfinished">অজানা নতুন নিয়ম সক্রিয় হলো (ভার্শনবিট %i)</translation>
-    </message>
-    <message>
-        <source>Verifying blocks…</source>
-        <translation type="unfinished">ব্লকস যাচাই করা হচ্ছে...</translation>
-    </message>
-    <message>
-        <source>Verifying wallet(s)…</source>
-        <translation type="unfinished">ওয়ালেট(স) যাচাই করা হচ্ছে...</translation>
-    </message>
-    </context>
 <context>
     <name>BitcoinGUI</name>
     <message>
         <source>Create a new wallet</source>
         <translation type="unfinished">একটি নতুন ওয়ালেট তৈরি করুন</translation>
     </message>
+    <message>
+        <source>&amp;Minimize</source>
+        <translation type="unfinished">ছোট করুন</translation>
+    </message>
     <message>
         <source>&amp;Options…</source>
         <translation type="unfinished">&amp;বিকল্প...</translation>
@@ -203,10 +217,6 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>Processing blocks on disk…</source>
         <translation type="unfinished">ডিস্কে ব্লক প্রসেস করা হচ্ছে...</translation>
     </message>
-    <message>
-        <source>Reindexing blocks on disk…</source>
-        <translation type="unfinished">ডিস্ক এ ব্লকস পুনর্বিন্যাস করা হচ্ছে...</translation>
-    </message>
     <message>
         <source>Connecting to peers…</source>
         <translation type="unfinished">সহকর্মীদের সাথে সংযোগ করা হচ্ছে...</translation>
@@ -246,7 +256,34 @@ Signing is only possible with addresses of the type 'legacy'.</source>
             <numerusform />
         </translation>
     </message>
-    </context>
+    <message>
+        <source>Click for more actions.</source>
+        <extracomment>A substring of the tooltip. "More actions" are available via the context menu.</extracomment>
+        <translation type="unfinished">আরো কর্মের জন্য চাপ দিন</translation>
+    </message>
+    <message>
+        <source>Disable network activity</source>
+        <extracomment>A context menu item.</extracomment>
+        <translation type="unfinished">নেটওয়ার্ক কার্যকলাপ বন্ধ করুন</translation>
+    </message>
+    <message>
+        <source>Enable network activity</source>
+        <extracomment>A context menu item. The network activity was disabled previously.</extracomment>
+        <translation type="unfinished">নেটওয়ার্ক কার্যকলাপ চালু করুন</translation>
+    </message>
+    <message>
+        <source>Sent transaction</source>
+        <translation type="unfinished">লেনদেন পাঠানো হয়েছে</translation>
+    </message>
+    <message>
+        <source>Incoming transaction</source>
+        <translation type="unfinished">লেনদেন আসছে</translation>
+    </message>
+    <message>
+        <source>Original message:</source>
+        <translation type="unfinished">আসল বার্তা:</translation>
+    </message>
+</context>
 <context>
     <name>UnitDisplayStatusBarControl</name>
     <message>
@@ -256,6 +293,22 @@ Signing is only possible with addresses of the type 'legacy'.</source>
 </context>
 <context>
     <name>CoinControlDialog</name>
+    <message>
+        <source>Coin Selection</source>
+        <translation type="unfinished">মুদ্রা নির্বাচন</translation>
+    </message>
+    <message>
+        <source>Quantity:</source>
+        <translation type="unfinished">পরিমাণ</translation>
+    </message>
+    <message>
+        <source>Fee:</source>
+        <translation type="unfinished">পারিশ্রমিক</translation>
+    </message>
+    <message>
+        <source>Change:</source>
+        <translation type="unfinished">পরিবর্তন</translation>
+    </message>
     <message>
         <source>Date</source>
         <translation type="unfinished">তারিখ</translation>
@@ -264,13 +317,25 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>Confirmed</source>
         <translation type="unfinished">নিশ্চিত করা হয়েছে</translation>
     </message>
+    <message>
+        <source>&amp;Copy address</source>
+        <translation type="unfinished">ঠিকানা কপি করুন</translation>
+    </message>
+    <message>
+        <source>Copy &amp;label</source>
+        <translation type="unfinished">কপি লেবেল</translation>
+    </message>
+    <message>
+        <source>Copy &amp;amount</source>
+        <translation type="unfinished">কপি পরিমাণ</translation>
+    </message>
+    <message>
+        <source>Copy transaction &amp;ID and output index</source>
+        <translation type="unfinished">লেনদেন আইডি এবং আউটপুট সূচক কপি করুন</translation>
+    </message>
     </context>
 <context>
     <name>WalletController</name>
-    <message>
-        <source>Close wallet</source>
-        <translation type="unfinished">ওয়ালেট বন্ধ করুন</translation>
-    </message>
     <message>
         <source>Are you sure you wish to close the wallet &lt;i&gt;%1&lt;/i&gt;?</source>
         <translation type="unfinished">আপনি কি নিশ্চিত যে আপনি ওয়ালেট বন্ধ করতে চান&lt;i&gt;%1&lt;/i&gt;?</translation>
@@ -310,6 +375,27 @@ Signing is only possible with addresses of the type 'legacy'.</source>
     </context>
 <context>
     <name>Intro</name>
+    <message numerus="yes">
+        <source>%n GB of space available</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>(of %n GB needed)</source>
+        <translation type="unfinished">
+            <numerusform>(of %n GB needed)</numerusform>
+            <numerusform>(of %n GB needed)</numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>(%n GB needed for full chain)</source>
+        <translation type="unfinished">
+            <numerusform>(%n GB needed for full chain)</numerusform>
+            <numerusform>(%n GB needed for full chain)</numerusform>
+        </translation>
+    </message>
     <message numerus="yes">
         <source>(sufficient to restore backups %n day(s) old)</source>
         <extracomment>Explanatory text on the capability of the current prune target.</extracomment>
@@ -344,6 +430,11 @@ Signing is only possible with addresses of the type 'legacy'.</source>
 </context>
 <context>
     <name>RPCConsole</name>
+    <message>
+        <source>&amp;Copy address</source>
+        <extracomment>Context menu action to copy the address of a peer.</extracomment>
+        <translation type="unfinished">ঠিকানা কপি করুন</translation>
+    </message>
     <message>
         <source>Unknown</source>
         <translation type="unfinished">অজানা</translation>
@@ -363,6 +454,18 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>&amp;Message:</source>
         <translation type="unfinished">&amp;বার্তাঃ</translation>
     </message>
+    <message>
+        <source>&amp;Copy address</source>
+        <translation type="unfinished">ঠিকানা কপি করুন</translation>
+    </message>
+    <message>
+        <source>Copy &amp;label</source>
+        <translation type="unfinished">কপি লেবেল</translation>
+    </message>
+    <message>
+        <source>Copy &amp;amount</source>
+        <translation type="unfinished">কপি পরিমাণ</translation>
+    </message>
     </context>
 <context>
     <name>RecentRequestsTableModel</name>
@@ -372,11 +475,23 @@ Signing is only possible with addresses of the type 'legacy'.</source>
     </message>
     <message>
         <source>Label</source>
-        <translation type="unfinished">লেবেল</translation>
+        <translation type="unfinished">টিকেট</translation>
     </message>
     </context>
 <context>
     <name>SendCoinsDialog</name>
+    <message>
+        <source>Quantity:</source>
+        <translation type="unfinished">পরিমাণ</translation>
+    </message>
+    <message>
+        <source>Fee:</source>
+        <translation type="unfinished">পারিশ্রমিক</translation>
+    </message>
+    <message>
+        <source>Change:</source>
+        <translation type="unfinished">পরিবর্তন</translation>
+    </message>
     <message numerus="yes">
         <source>Estimated to begin confirmation within %n block(s).</source>
         <translation type="unfinished">
@@ -418,11 +533,23 @@ Signing is only possible with addresses of the type 'legacy'.</source>
     </message>
     <message>
         <source>Label</source>
-        <translation type="unfinished">লেবেল</translation>
+        <translation type="unfinished">টিকেট</translation>
     </message>
     </context>
 <context>
     <name>TransactionView</name>
+    <message>
+        <source>&amp;Copy address</source>
+        <translation type="unfinished">ঠিকানা কপি করুন</translation>
+    </message>
+    <message>
+        <source>Copy &amp;label</source>
+        <translation type="unfinished">কপি লেবেল</translation>
+    </message>
+    <message>
+        <source>Copy &amp;amount</source>
+        <translation type="unfinished">কপি পরিমাণ</translation>
+    </message>
     <message>
         <source>Comma separated file</source>
         <extracomment>Expanded name of the CSV file format. See: https://en.wikipedia.org/wiki/Comma-separated_values.</extracomment>
@@ -446,7 +573,7 @@ Signing is only possible with addresses of the type 'legacy'.</source>
     </message>
     <message>
         <source>Label</source>
-        <translation type="unfinished">লেবেল</translation>
+        <translation type="unfinished">টিকেট</translation>
     </message>
     <message>
         <source>Address</source>
@@ -480,4 +607,46 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <translation type="unfinished">একটি নতুন ওয়ালেট তৈরি করুন</translation>
     </message>
     </context>
+<context>
+    <name>WalletView</name>
+    <message>
+        <source>Export the data in the current tab to a file</source>
+        <translation type="unfinished">বর্তমান ট্যাবের তথ্যগুলো একটি আলাদা নথিতে লিপিবদ্ধ করুন </translation>
+    </message>
+    </context>
+<context>
+    <name>bitcoin-core</name>
+    <message>
+        <source>SQLiteDatabase: Unexpected application id. Expected %u, got %u</source>
+        <translation type="unfinished">এস. কিয়ু. লাইট ডাটাবেস : অপ্রত্যাশিত এপ্লিকেশন আই.ডি. প্রত্যাশিত %u, পাওয়া গেলো %u </translation>
+    </message>
+    <message>
+        <source>Starting network threads…</source>
+        <translation type="unfinished">নেটওয়ার্ক থ্রেড শুরু হচ্ছে...</translation>
+    </message>
+    <message>
+        <source>The specified config file %s does not exist</source>
+        <translation type="unfinished">নির্দিষ্ট কনফিগ ফাইল %s এর অস্তিত্ব নেই</translation>
+    </message>
+    <message>
+        <source>Unable to open %s for writing</source>
+        <translation type="unfinished">লেখার জন্যে %s খোলা যাচ্ছে না</translation>
+    </message>
+    <message>
+        <source>Unknown new rules activated (versionbit %i)</source>
+        <translation type="unfinished">অজানা নতুন নিয়ম সক্রিয় হলো (ভার্শনবিট %i)</translation>
+    </message>
+    <message>
+        <source>Verifying blocks…</source>
+        <translation type="unfinished">ব্লকস যাচাই করা হচ্ছে...</translation>
+    </message>
+    <message>
+        <source>Verifying wallet(s)…</source>
+        <translation type="unfinished">ওয়ালেট(স) যাচাই করা হচ্ছে...</translation>
+    </message>
+    <message>
+        <source>Settings file could not be read</source>
+        <translation type="unfinished">Settingsসেটিংস ফাইল পড়া যাবে না।fileসেটিংস ফাইল পড়া যাবে না।couldসেটিংস ফাইল পড়া যাবে না।notসেটিংস ফাইল পড়া যাবে না।beসেটিংস ফাইল পড়া যাবে না।read</translation>
+    </message>
+    </context>
 </TS>
\ No newline at end of file
