@@ -91,6 +91,11 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Export Address List</source>
         <translation type="unfinished">Exportar Lista de Enderezos</translation>
     </message>
+    <message>
+        <source>There was an error trying to save the address list to %1. Please try again.</source>
+        <extracomment>An error message. %1 is a stand-in argument for the name of the file we attempted to save to.</extracomment>
+        <translation type="unfinished">Houbo un erro tentando gardar a lista de enderezos en %1. Por favor proba de novo.</translation>
+    </message>
     <message>
         <source>Exporting Failed</source>
         <translation type="unfinished">Exportación falida</translation>
@@ -129,6 +134,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Repeat new passphrase</source>
         <translation type="unfinished">Repite novo contrasinal</translation>
     </message>
+    <message>
+        <source>Show passphrase</source>
+        <translation type="unfinished">Mostra frase contrasinal</translation>
+    </message>
     <message>
         <source>Encrypt wallet</source>
         <translation type="unfinished">Encriptar moedeiro</translation>
@@ -161,6 +170,30 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Wallet encrypted</source>
         <translation type="unfinished">Moedeiro encriptado</translation>
     </message>
+    <message>
+        <source>Enter the new passphrase for the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
+        <translation type="unfinished">Introduce unha nova frase contrasinal para a carteira.&lt;br/&gt;Por favor utiliza una frase contrasinal que &lt;b&gt;teña dez ou máis caracteres aleatorios&lt;/b&gt;, ou &lt;b&gt;oito ou máis palabras&lt;/b&gt;.</translation>
+    </message>
+    <message>
+        <source>Enter the old passphrase and new passphrase for the wallet.</source>
+        <translation type="unfinished">Introduce a frase contrasinal anterior mais a nova frase contrasinal para a carteira.</translation>
+    </message>
+    <message>
+        <source>Remember that encrypting your wallet cannot fully protect your bitcoins from being stolen by malware infecting your computer.</source>
+        <translation type="unfinished">Recorda que encriptar a tua carteira non protexe completamente que os teus bitcoins poidan ser roubados por malware que afecte ó teu computador.</translation>
+    </message>
+    <message>
+        <source>Wallet to be encrypted</source>
+        <translation type="unfinished">Carteira para ser encriptada</translation>
+    </message>
+    <message>
+        <source>Your wallet is about to be encrypted. </source>
+        <translation type="unfinished">A túa carteira vai a ser encriptada.</translation>
+    </message>
+    <message>
+        <source>Your wallet is now encrypted. </source>
+        <translation type="unfinished">A túa carteira está agora encriptada.</translation>
+    </message>
     <message>
         <source>IMPORTANT: Any previous backups you have made of your wallet file should be replaced with the newly generated, encrypted wallet file. For security reasons, previous backups of the unencrypted wallet file will become useless as soon as you start using the new, encrypted wallet.</source>
         <translation type="unfinished">IMPORTANTE: Calquera copia de seguridade previa que fixeses do teu arquivo de moedeiro debería ser substituída polo recén xerado arquivo encriptado de moedeiro. Por razóns de seguridade, as copias de seguridade previas de un arquivo de moedeiro desencriptado tornaránse inútiles no momento no que comeces a emprega-lo novo, encriptado, moedeiro.</translation>
@@ -195,11 +228,25 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
     </message>
 </context>
 <context>
-    <name>QObject</name>
+    <name>BanTableModel</name>
     <message>
-        <source>Error: Specified data directory "%1" does not exist.</source>
-        <translation type="unfinished">Erro: O directorio de datos especificado "%1" non existe.</translation>
+        <source>IP/Netmask</source>
+        <translation type="unfinished">IP/Máscara de rede</translation>
     </message>
+    <message>
+        <source>Banned Until</source>
+        <translation type="unfinished">Vedado ata</translation>
+    </message>
+</context>
+<context>
+    <name>BitcoinApplication</name>
+    <message>
+        <source>Internal error</source>
+        <translation type="unfinished">Erro interno</translation>
+    </message>
+    </context>
+<context>
+    <name>QObject</name>
     <message>
         <source>unknown</source>
         <translation type="unfinished">descoñecido</translation>
@@ -251,77 +298,6 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         </translation>
     </message>
     </context>
-<context>
-    <name>bitcoin-core</name>
-    <message>
-        <source>This is a pre-release test build - use at your own risk - do not use for mining or merchant applications</source>
-        <translation type="unfinished">Esta é unha build de test pre-lanzamento - emprégaa baixo o teu propio risco - non empregar para minado ou aplicacións de comerciantes</translation>
-    </message>
-    <message>
-        <source>Warning: We do not appear to fully agree with our peers! You may need to upgrade, or other nodes may need to upgrade.</source>
-        <translation type="unfinished">Precaución: Non parece que esteamos totalmente de acordo cos nosos pares! Pode que precises actualizar, ou outros nodos poden precisar actualizarse.</translation>
-    </message>
-    <message>
-        <source>Corrupted block database detected</source>
-        <translation type="unfinished">Detectada base de datos de bloques corrupta.</translation>
-    </message>
-    <message>
-        <source>Do you want to rebuild the block database now?</source>
-        <translation type="unfinished">Queres reconstruír a base de datos de bloques agora?</translation>
-    </message>
-    <message>
-        <source>Done loading</source>
-        <translation type="unfinished">Carga completa</translation>
-    </message>
-    <message>
-        <source>Error initializing block database</source>
-        <translation type="unfinished">Erro inicializando a base de datos de bloques</translation>
-    </message>
-    <message>
-        <source>Error initializing wallet database environment %s!</source>
-        <translation type="unfinished">Erro inicializando entorno de base de datos de moedeiro %s!</translation>
-    </message>
-    <message>
-        <source>Error loading block database</source>
-        <translation type="unfinished">Erro cargando base de datos do bloque</translation>
-    </message>
-    <message>
-        <source>Error opening block database</source>
-        <translation type="unfinished">Erro abrindo base de datos de bloques</translation>
-    </message>
-    <message>
-        <source>Failed to listen on any port. Use -listen=0 if you want this.</source>
-        <translation type="unfinished">Fallou escoitar en calquera porto. Emprega -listen=0 se queres isto.</translation>
-    </message>
-    <message>
-        <source>Incorrect or no genesis block found. Wrong datadir for network?</source>
-        <translation type="unfinished">Bloque xénese incorrecto ou non existente. Datadir erróneo para a rede?</translation>
-    </message>
-    <message>
-        <source>Insufficient funds</source>
-        <translation type="unfinished">Fondos insuficientes</translation>
-    </message>
-    <message>
-        <source>Not enough file descriptors available.</source>
-        <translation type="unfinished">Non hai suficientes descritores de arquivo dispoñibles.</translation>
-    </message>
-    <message>
-        <source>Signing transaction failed</source>
-        <translation type="unfinished">Fallou a sinatura da transacción</translation>
-    </message>
-    <message>
-        <source>Transaction amount too small</source>
-        <translation type="unfinished">A cantidade da transacción é demasiado pequena</translation>
-    </message>
-    <message>
-        <source>Transaction too large</source>
-        <translation type="unfinished">A transacción é demasiado grande</translation>
-    </message>
-    <message>
-        <source>Unknown network specified in -onlynet: '%s'</source>
-        <translation type="unfinished">Rede descoñecida especificada en -onlynet: '%s'</translation>
-    </message>
-    </context>
 <context>
     <name>BitcoinGUI</name>
     <message>
@@ -348,6 +324,14 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Quit application</source>
         <translation type="unfinished">Saír da aplicación</translation>
     </message>
+    <message>
+        <source>&amp;About %1</source>
+        <translation type="unfinished">&amp;A cerca de %1</translation>
+    </message>
+    <message>
+        <source>Show information about %1</source>
+        <translation type="unfinished">Mostra información acerca de %1</translation>
+    </message>
     <message>
         <source>About &amp;Qt</source>
         <translation type="unfinished">Acerca de &amp;Qt</translation>
@@ -356,10 +340,31 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Show information about Qt</source>
         <translation type="unfinished">Amosar información acerca de Qt</translation>
     </message>
+    <message>
+        <source>Modify configuration options for %1</source>
+        <translation type="unfinished">Modifica as opcións de configuración de %1</translation>
+    </message>
+    <message>
+        <source>Create a new wallet</source>
+        <translation type="unfinished">Crear unha nova carteira</translation>
+    </message>
+    <message>
+        <source>&amp;Minimize</source>
+        <translation type="unfinished">&amp;Minimizar</translation>
+    </message>
     <message>
         <source>Wallet:</source>
         <translation type="unfinished">Moedeiro:</translation>
     </message>
+    <message>
+        <source>Network activity disabled.</source>
+        <extracomment>A substring of the tooltip.</extracomment>
+        <translation type="unfinished">Actividade da rede desactivada.</translation>
+    </message>
+    <message>
+        <source>Proxy is &lt;b&gt;enabled&lt;/b&gt;: %1</source>
+        <translation type="unfinished">Proxy &lt;b&gt;activado&lt;/b&gt;: %1</translation>
+    </message>
     <message>
         <source>Send coins to a Bitcoin address</source>
         <translation type="unfinished">Enviar moedas a unha dirección Bitcoin</translation>
@@ -380,6 +385,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>&amp;Receive</source>
         <translation type="unfinished">&amp;Recibir</translation>
     </message>
+    <message>
+        <source>&amp;Options…</source>
+        <translation type="unfinished">&amp;Opcións...</translation>
+    </message>
     <message>
         <source>Encrypt the private keys that belong to your wallet</source>
         <translation type="unfinished">Encriptar as claves privadas que pertencen ao teu moedeiro</translation>
@@ -388,6 +397,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Sign messages with your Bitcoin addresses to prove you own them</source>
         <translation type="unfinished">Asina mensaxes cos teus enderezos Bitcoin para probar que che pertencen</translation>
     </message>
+    <message>
+        <source>&amp;Verify message…</source>
+        <translation type="unfinished">&amp;Verifica a mensaxe...</translation>
+    </message>
     <message>
         <source>Verify messages to ensure they were signed with specified Bitcoin addresses</source>
         <translation type="unfinished">Verifica mensaxes para asegurar que foron asinados con enderezos Bitcoin específicos.</translation>
@@ -459,6 +472,26 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Up to date</source>
         <translation type="unfinished">Actualizado</translation>
     </message>
+    <message>
+        <source>Node window</source>
+        <translation type="unfinished">Xanela de Nodo</translation>
+    </message>
+    <message>
+        <source>Open node debugging and diagnostic console</source>
+        <translation type="unfinished">Abre a consola de depuración e diagnostico do nodo</translation>
+    </message>
+    <message>
+        <source>&amp;Sending addresses</source>
+        <translation type="unfinished">&amp;Enderezos de envío</translation>
+    </message>
+    <message>
+        <source>&amp;Receiving addresses</source>
+        <translation type="unfinished">&amp;Enderezos de recepción</translation>
+    </message>
+    <message>
+        <source>Open a bitcoin: URI</source>
+        <translation type="unfinished">Abre una URI de Bitcoin</translation>
+    </message>
     <message>
         <source>Open Wallet</source>
         <translation type="unfinished">Abrir Moedeiro</translation>
@@ -471,6 +504,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Close wallet</source>
         <translation type="unfinished">Pechar moedeiro</translation>
     </message>
+    <message>
+        <source>Show the %1 help message to get a list with possible Bitcoin command-line options</source>
+        <translation type="unfinished">Mostra a %1 mensaxe de axuda para obter unha lista cas posibles opcións de línea de comando de Bitcoin </translation>
+    </message>
     <message>
         <source>default wallet</source>
         <translation type="unfinished">moedeiro por defecto</translation>
@@ -479,6 +516,11 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>No wallets available</source>
         <translation type="unfinished">Non hai moedeiros dispoñíbeis</translation>
     </message>
+    <message>
+        <source>Wallet Name</source>
+        <extracomment>Label of the input field where the name of the wallet is entered.</extracomment>
+        <translation type="unfinished">Nome da Carteira</translation>
+    </message>
     <message>
         <source>&amp;Window</source>
         <translation type="unfinished">&amp;Xanela</translation>
@@ -499,6 +541,46 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
             <numerusform />
         </translation>
     </message>
+    <message>
+        <source>Warning: %1</source>
+        <translation type="unfinished">Aviso: %1</translation>
+    </message>
+    <message>
+        <source>Date: %1
+</source>
+        <translation type="unfinished">Data: %1
+</translation>
+    </message>
+    <message>
+        <source>Amount: %1
+</source>
+        <translation type="unfinished">Cantidade: %1
+</translation>
+    </message>
+    <message>
+        <source>Wallet: %1
+</source>
+        <translation type="unfinished">Carteira: %1
+</translation>
+    </message>
+    <message>
+        <source>Type: %1
+</source>
+        <translation type="unfinished">Escribe: %1
+</translation>
+    </message>
+    <message>
+        <source>Label: %1
+</source>
+        <translation type="unfinished">Etiqueta: %1
+</translation>
+    </message>
+    <message>
+        <source>Address: %1
+</source>
+        <translation type="unfinished">Enderezo: %1
+</translation>
+    </message>
     <message>
         <source>Sent transaction</source>
         <translation type="unfinished">Transacción enviada</translation>
@@ -507,6 +589,18 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Incoming transaction</source>
         <translation type="unfinished">Transacción entrante</translation>
     </message>
+    <message>
+        <source>HD key generation is &lt;b&gt;enabled&lt;/b&gt;</source>
+        <translation type="unfinished">A xeración de clave HD está &lt;b&gt;activada&lt;/b&gt;</translation>
+    </message>
+    <message>
+        <source>HD key generation is &lt;b&gt;disabled&lt;/b&gt;</source>
+        <translation type="unfinished">A xeración de clave HD está &lt;b&gt;desactivada&lt;/b&gt;</translation>
+    </message>
+    <message>
+        <source>Private key &lt;b&gt;disabled&lt;/b&gt;</source>
+        <translation type="unfinished">Clave privada &lt;b&gt;desactivada&lt;/b&gt;</translation>
+    </message>
     <message>
         <source>Wallet is &lt;b&gt;encrypted&lt;/b&gt; and currently &lt;b&gt;unlocked&lt;/b&gt;</source>
         <translation type="unfinished">O moedeiro está &lt;b&gt;encriptado&lt;/b&gt; e actualmente &lt;b&gt;desbloqueado&lt;/b&gt;</translation>
@@ -515,9 +609,17 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Wallet is &lt;b&gt;encrypted&lt;/b&gt; and currently &lt;b&gt;locked&lt;/b&gt;</source>
         <translation type="unfinished">O moedeiro está &lt;b&gt;encriptado&lt;/b&gt; e actualmente &lt;b&gt;bloqueado&lt;/b&gt;</translation>
     </message>
-    </context>
+    <message>
+        <source>Original message:</source>
+        <translation type="unfinished">Mensaxe orixinal:</translation>
+    </message>
+</context>
 <context>
     <name>CoinControlDialog</name>
+    <message>
+        <source>Coin Selection</source>
+        <translation type="unfinished">Selección de moeda</translation>
+    </message>
     <message>
         <source>Quantity:</source>
         <translation type="unfinished">Cantidade:</translation>
@@ -530,6 +632,14 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Fee:</source>
         <translation type="unfinished">Taxa:</translation>
     </message>
+    <message>
+        <source>Dust:</source>
+        <translation type="unfinished">po:</translation>
+    </message>
+    <message>
+        <source>After Fee:</source>
+        <translation type="unfinished">Despois de taxas:</translation>
+    </message>
     <message>
         <source>Change:</source>
         <translation type="unfinished">Cambiar:</translation>
@@ -550,6 +660,14 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Amount</source>
         <translation type="unfinished">Cantidade</translation>
     </message>
+    <message>
+        <source>Received with label</source>
+        <translation type="unfinished">Recibida con etiqueta</translation>
+    </message>
+    <message>
+        <source>Received with address</source>
+        <translation type="unfinished">Recibida con enderezo</translation>
+    </message>
     <message>
         <source>Date</source>
         <translation type="unfinished">Data</translation>
@@ -566,6 +684,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Copy amount</source>
         <translation type="unfinished">Copiar cantidade</translation>
     </message>
+    <message>
+        <source>&amp;Copy address</source>
+        <translation type="unfinished">&amp;Copiar enderezo</translation>
+    </message>
     <message>
         <source>Copy quantity</source>
         <translation type="unfinished">Copiar cantidade</translation>
@@ -582,6 +704,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Copy bytes</source>
         <translation type="unfinished">Copiar bytes</translation>
     </message>
+    <message>
+        <source>Copy dust</source>
+        <translation type="unfinished">Copiar po</translation>
+    </message>
     <message>
         <source>Copy change</source>
         <translation type="unfinished">Copiar cambio</translation>
@@ -598,15 +724,43 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>no</source>
         <translation type="unfinished">non</translation>
     </message>
+    <message>
+        <source>This label turns red if any recipient receives an amount smaller than the current dust threshold.</source>
+        <translation type="unfinished">Esta etiqueta tórnase vermella se algún receptor recibe unha cantidade máis pequena que o actual límite de po.</translation>
+    </message>
+    <message>
+        <source>Can vary +/- %1 satoshi(s) per input.</source>
+        <translation type="unfinished">Pode variar +/- %1 satoshi(s) por entrada.</translation>
+    </message>
     <message>
         <source>(no label)</source>
         <translation type="unfinished">(sen etiqueta)</translation>
     </message>
+    <message>
+        <source>change from %1 (%2)</source>
+        <translation type="unfinished">Cambia de %1 a (%2)</translation>
+    </message>
     <message>
         <source>(change)</source>
         <translation type="unfinished">(cambio)</translation>
     </message>
 </context>
+<context>
+    <name>CreateWalletActivity</name>
+    <message>
+        <source>Create Wallet</source>
+        <extracomment>Title of window indicating the progress of creation of a new wallet.</extracomment>
+        <translation type="unfinished">Crea unha Carteira</translation>
+    </message>
+    <message>
+        <source>Create wallet failed</source>
+        <translation type="unfinished">Creación de carteira fallida</translation>
+    </message>
+    <message>
+        <source>Create wallet warning</source>
+        <translation type="unfinished">Creación de carteira con aviso</translation>
+    </message>
+    </context>
 <context>
     <name>OpenWalletActivity</name>
     <message>
@@ -628,10 +782,50 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
     </context>
 <context>
     <name>CreateWalletDialog</name>
+    <message>
+        <source>Create Wallet</source>
+        <translation type="unfinished">Crea unha Carteira</translation>
+    </message>
+    <message>
+        <source>Wallet Name</source>
+        <translation type="unfinished">Nome da Carteira</translation>
+    </message>
     <message>
         <source>Wallet</source>
         <translation type="unfinished">Moedeiro</translation>
     </message>
+    <message>
+        <source>Encrypt the wallet. The wallet will be encrypted with a passphrase of your choice.</source>
+        <translation type="unfinished">Encripta a carteira. A carteira sera encriptada cunha frase contrasinal que tú elixas.</translation>
+    </message>
+    <message>
+        <source>Encrypt Wallet</source>
+        <translation type="unfinished">Encriptar Carteira</translation>
+    </message>
+    <message>
+        <source>Advanced Options</source>
+        <translation type="unfinished">Opcións avanzadas</translation>
+    </message>
+    <message>
+        <source>Disable private keys for this wallet. Wallets with private keys disabled will have no private keys and cannot have an HD seed or imported private keys. This is ideal for watch-only wallets.</source>
+        <translation type="unfinished">Desactiva as claves privadas para esta carteira. Carteiras con claves privadas desactivadas non terán claves privadas e polo tanto non poderan ter unha semente HD ou claves privadas importadas. Esto é ideal para carteiras de solo visualización.</translation>
+    </message>
+    <message>
+        <source>Disable Private Keys</source>
+        <translation type="unfinished">Desactivar Claves Privadas</translation>
+    </message>
+    <message>
+        <source>Make a blank wallet. Blank wallets do not initially have private keys or scripts. Private keys and addresses can be imported, or an HD seed can be set, at a later time.</source>
+        <translation type="unfinished">Crear unha Carteira en blanco. As carteiras en blanco non teñen inicialmente claves privadas ou scripts. As claves privadas poden ser importadas ou unha semente HD poder ser configurada, máis adiante.</translation>
+    </message>
+    <message>
+        <source>Make Blank Wallet</source>
+        <translation type="unfinished">Crea unha Carteira en Blanco</translation>
+    </message>
+    <message>
+        <source>Create</source>
+        <translation type="unfinished">Crea</translation>
+    </message>
     </context>
 <context>
     <name>EditAddressDialog</name>
@@ -671,6 +865,14 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>The entered address "%1" is not a valid Bitcoin address.</source>
         <translation type="unfinished">A dirección introducida '%1' non é unha dirección Bitcoin válida.</translation>
     </message>
+    <message>
+        <source>Address "%1" already exists as a receiving address with label "%2" and so cannot be added as a sending address.</source>
+        <translation type="unfinished">O enderezo "%1" xa existe como un enderezo de recepción ca etiqueta "%2" polo que non pode ser añadido como un enderezo de envío.</translation>
+    </message>
+    <message>
+        <source>The entered address "%1" is already in the address book with label "%2".</source>
+        <translation type="unfinished">O enderezo introducido "%1" xa existe na axenda de enderezos ca etiqueta "%2".</translation>
+    </message>
     <message>
         <source>Could not unlock wallet.</source>
         <translation type="unfinished">Non se puido desbloquear o moedeiro.</translation>
@@ -705,6 +907,27 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
 </context>
 <context>
     <name>Intro</name>
+    <message numerus="yes">
+        <source>%n GB of space available</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>(of %n GB needed)</source>
+        <translation type="unfinished">
+            <numerusform>(of %n GB needed)</numerusform>
+            <numerusform>(of %n GB needed)</numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>(%n GB needed for full chain)</source>
+        <translation type="unfinished">
+            <numerusform>(%n GB needed for full chain)</numerusform>
+            <numerusform>(%n GB needed for full chain)</numerusform>
+        </translation>
+    </message>
     <message numerus="yes">
         <source>(sufficient to restore backups %n day(s) old)</source>
         <extracomment>Explanatory text on the capability of the current prune target.</extracomment>
@@ -751,6 +974,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Form</source>
         <translation type="unfinished">Formulario</translation>
     </message>
+    <message>
+        <source>Unknown…</source>
+        <translation type="unfinished">Descoñecido...</translation>
+    </message>
     <message>
         <source>Last block time</source>
         <translation type="unfinished">Hora do último bloque</translation>
@@ -856,6 +1083,7 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
     </message>
     <message>
         <source>Confirm options reset</source>
+        <extracomment>Window title text of pop-up window shown when the user has chosen to reset options.</extracomment>
         <translation type="unfinished">Confirmar opcións de restaurar</translation>
     </message>
     <message>
@@ -863,6 +1091,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <extracomment>Window title text of pop-up box that allows opening up of configuration file.</extracomment>
         <translation type="unfinished">Opcións de configuración</translation>
     </message>
+    <message>
+        <source>Continue</source>
+        <translation type="unfinished">Continuar</translation>
+    </message>
     <message>
         <source>Error</source>
         <translation type="unfinished">Erro</translation>
@@ -915,6 +1147,17 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <translation type="unfinished">Transaccións recentes</translation>
     </message>
     </context>
+<context>
+    <name>PSBTOperationsDialog</name>
+    <message>
+        <source>Save…</source>
+        <translation type="unfinished">Gardar...</translation>
+    </message>
+    <message>
+        <source>Close</source>
+        <translation type="unfinished">Pechar</translation>
+    </message>
+    </context>
 <context>
     <name>PaymentServer</name>
     <message>
@@ -946,6 +1189,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
     </context>
 <context>
     <name>QRImageWidget</name>
+    <message>
+        <source>&amp;Save Image…</source>
+        <translation type="unfinished">&amp;Gardar Imaxe...</translation>
+    </message>
     <message>
         <source>&amp;Copy Image</source>
         <translation type="unfinished">&amp;Copiar Imaxe</translation>
@@ -989,6 +1236,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Block chain</source>
         <translation type="unfinished">Cadea de bloques</translation>
     </message>
+    <message>
+        <source>Node window</source>
+        <translation type="unfinished">Xanela de Nodo</translation>
+    </message>
     <message>
         <source>Last block time</source>
         <translation type="unfinished">Hora do último bloque</translation>
@@ -1025,6 +1276,11 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Out:</source>
         <translation type="unfinished">Fóra:</translation>
     </message>
+    <message>
+        <source>&amp;Copy address</source>
+        <extracomment>Context menu action to copy the address of a peer.</extracomment>
+        <translation type="unfinished">&amp;Copiar enderezo</translation>
+    </message>
     <message>
         <source>To</source>
         <translation type="unfinished">A</translation>
@@ -1064,6 +1320,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Copy &amp;URI</source>
         <translation type="unfinished">Copiar &amp;URI</translation>
     </message>
+    <message>
+        <source>&amp;Copy address</source>
+        <translation type="unfinished">&amp;Copiar enderezo</translation>
+    </message>
     <message>
         <source>Could not unlock wallet.</source>
         <translation type="unfinished">Non se puido desbloquear o moedeiro.</translation>
@@ -1091,6 +1351,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Copy &amp;Address</source>
         <translation type="unfinished">Copiar &amp;Enderezo</translation>
     </message>
+    <message>
+        <source>&amp;Save Image…</source>
+        <translation type="unfinished">&amp;Gardar Imaxe...</translation>
+    </message>
     <message>
         <source>Payment information</source>
         <translation type="unfinished">Información de Pago</translation>
@@ -1141,6 +1405,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Fee:</source>
         <translation type="unfinished">Taxa:</translation>
     </message>
+    <message>
+        <source>After Fee:</source>
+        <translation type="unfinished">Despois de taxas:</translation>
+    </message>
     <message>
         <source>Change:</source>
         <translation type="unfinished">Cambiar:</translation>
@@ -1161,6 +1429,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Clear all fields of the form.</source>
         <translation type="unfinished">Limpar tódolos campos do formulario</translation>
     </message>
+    <message>
+        <source>Dust:</source>
+        <translation type="unfinished">po:</translation>
+    </message>
     <message>
         <source>Clear &amp;All</source>
         <translation type="unfinished">Limpar &amp;Todo</translation>
@@ -1193,6 +1465,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Copy bytes</source>
         <translation type="unfinished">Copiar bytes</translation>
     </message>
+    <message>
+        <source>Copy dust</source>
+        <translation type="unfinished">Copiar po</translation>
+    </message>
     <message>
         <source>Copy change</source>
         <translation type="unfinished">Copiar cambio</translation>
@@ -1221,10 +1497,6 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>The total exceeds your balance when the %1 transaction fee is included.</source>
         <translation type="unfinished">O total sobrepasa o teu balance cando se inclúe a tarifa de transacción %1.</translation>
     </message>
-    <message>
-        <source>Payment request expired.</source>
-        <translation type="unfinished">Solicitude de pagamento expirada.</translation>
-    </message>
     <message numerus="yes">
         <source>Estimated to begin confirmation within %n block(s).</source>
         <translation type="unfinished">
@@ -1279,10 +1551,6 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Enter a label for this address to add it to the list of used addresses</source>
         <translation type="unfinished">Introduce unha etiqueta para esta dirección para engadila á listaxe de direccións empregadas</translation>
     </message>
-    <message>
-        <source>Pay To:</source>
-        <translation type="unfinished">Pagar A:</translation>
-    </message>
     </context>
 <context>
     <name>SignVerifyMessageDialog</name>
@@ -1403,10 +1671,12 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
     <name>TransactionDesc</name>
     <message>
         <source>%1/unconfirmed</source>
+        <extracomment>Text explaining the current status of a transaction, shown in the status field of the details window for this transaction. This status represents a transaction confirmed in at least one block, but less than 6 blocks.</extracomment>
         <translation type="unfinished">%1/sen confirmar</translation>
     </message>
     <message>
         <source>%1 confirmations</source>
+        <extracomment>Text explaining the current status of a transaction, shown in the status field of the details window for this transaction. This status represents a transaction confirmed in 6 or more blocks.</extracomment>
         <translation type="unfinished">%1 confirmacións</translation>
     </message>
     <message>
@@ -1637,6 +1907,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <source>Min amount</source>
         <translation type="unfinished">Cantidade mínima</translation>
     </message>
+    <message>
+        <source>&amp;Copy address</source>
+        <translation type="unfinished">&amp;Copiar enderezo</translation>
+    </message>
     <message>
         <source>Export Transaction History</source>
         <translation type="unfinished">Exportar Historial de Transaccións</translation>
@@ -1688,6 +1962,10 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
 </context>
 <context>
     <name>WalletFrame</name>
+    <message>
+        <source>Create a new wallet</source>
+        <translation type="unfinished">Crear unha nova carteira</translation>
+    </message>
     <message>
         <source>Error</source>
         <translation type="unfinished">Erro</translation>
@@ -1735,4 +2013,75 @@ Firmar é posible unicamente con enderezos de tipo 'legacy'.</translation>
         <translation type="unfinished">Os datos do moedeiro foron gardados correctamente en %1.</translation>
     </message>
     </context>
+<context>
+    <name>bitcoin-core</name>
+    <message>
+        <source>This is a pre-release test build - use at your own risk - do not use for mining or merchant applications</source>
+        <translation type="unfinished">Esta é unha build de test pre-lanzamento - emprégaa baixo o teu propio risco - non empregar para minado ou aplicacións de comerciantes</translation>
+    </message>
+    <message>
+        <source>Warning: We do not appear to fully agree with our peers! You may need to upgrade, or other nodes may need to upgrade.</source>
+        <translation type="unfinished">Precaución: Non parece que esteamos totalmente de acordo cos nosos pares! Pode que precises actualizar, ou outros nodos poden precisar actualizarse.</translation>
+    </message>
+    <message>
+        <source>Corrupted block database detected</source>
+        <translation type="unfinished">Detectada base de datos de bloques corrupta.</translation>
+    </message>
+    <message>
+        <source>Do you want to rebuild the block database now?</source>
+        <translation type="unfinished">Queres reconstruír a base de datos de bloques agora?</translation>
+    </message>
+    <message>
+        <source>Done loading</source>
+        <translation type="unfinished">Carga completa</translation>
+    </message>
+    <message>
+        <source>Error initializing block database</source>
+        <translation type="unfinished">Erro inicializando a base de datos de bloques</translation>
+    </message>
+    <message>
+        <source>Error initializing wallet database environment %s!</source>
+        <translation type="unfinished">Erro inicializando entorno de base de datos de moedeiro %s!</translation>
+    </message>
+    <message>
+        <source>Error loading block database</source>
+        <translation type="unfinished">Erro cargando base de datos do bloque</translation>
+    </message>
+    <message>
+        <source>Error opening block database</source>
+        <translation type="unfinished">Erro abrindo base de datos de bloques</translation>
+    </message>
+    <message>
+        <source>Failed to listen on any port. Use -listen=0 if you want this.</source>
+        <translation type="unfinished">Fallou escoitar en calquera porto. Emprega -listen=0 se queres isto.</translation>
+    </message>
+    <message>
+        <source>Incorrect or no genesis block found. Wrong datadir for network?</source>
+        <translation type="unfinished">Bloque xénese incorrecto ou non existente. Datadir erróneo para a rede?</translation>
+    </message>
+    <message>
+        <source>Insufficient funds</source>
+        <translation type="unfinished">Fondos insuficientes</translation>
+    </message>
+    <message>
+        <source>Not enough file descriptors available.</source>
+        <translation type="unfinished">Non hai suficientes descritores de arquivo dispoñibles.</translation>
+    </message>
+    <message>
+        <source>Signing transaction failed</source>
+        <translation type="unfinished">Fallou a sinatura da transacción</translation>
+    </message>
+    <message>
+        <source>Transaction amount too small</source>
+        <translation type="unfinished">A cantidade da transacción é demasiado pequena</translation>
+    </message>
+    <message>
+        <source>Transaction too large</source>
+        <translation type="unfinished">A transacción é demasiado grande</translation>
+    </message>
+    <message>
+        <source>Unknown network specified in -onlynet: '%s'</source>
+        <translation type="unfinished">Rede descoñecida especificada en -onlynet: '%s'</translation>
+    </message>
+    </context>
 </TS>
\ No newline at end of file
