@@ -81,7 +81,11 @@
         <source>Export Address List</source>
         <translation type="unfinished">Экспорт хийх хаягуудын жагсаалт</translation>
     </message>
-    </context>
+    <message>
+        <source>Exporting Failed</source>
+        <translation type="unfinished">Экспорт амжилтгүй боллоо</translation>
+    </message>
+</context>
 <context>
     <name>AddressTableModel</name>
     <message>
@@ -160,6 +164,13 @@
         <translation type="unfinished">Түрүйвчийн нууц үг амжилттай ѳѳр</translation>
     </message>
     </context>
+<context>
+    <name>BitcoinApplication</name>
+    <message>
+        <source>Internal error</source>
+        <translation type="unfinished">Дотоод алдаа</translation>
+    </message>
+    </context>
 <context>
     <name>QObject</name>
     <message>
@@ -221,17 +232,6 @@
         </translation>
     </message>
     </context>
-<context>
-    <name>bitcoin-core</name>
-    <message>
-        <source>Done loading</source>
-        <translation type="unfinished">Ачааллаж дууслаа</translation>
-    </message>
-    <message>
-        <source>Insufficient funds</source>
-        <translation type="unfinished">Таны дансны үлдэгдэл хүрэлцэхгүй байна</translation>
-    </message>
-    </context>
 <context>
     <name>BitcoinGUI</name>
     <message>
@@ -258,10 +258,19 @@
         <source>Show information about Qt</source>
         <translation type="unfinished">Клиентийн тухай мэдээллийг харуул</translation>
     </message>
+    <message>
+        <source>Create a new wallet</source>
+        <translation type="unfinished">Шинэ түрийвч үүсгэх</translation>
+    </message>
     <message>
         <source>Wallet:</source>
         <translation type="unfinished">Хэтэвч:</translation>
     </message>
+    <message>
+        <source>Network activity disabled.</source>
+        <extracomment>A substring of the tooltip.</extracomment>
+        <translation type="unfinished">Сүлжээний үйл ажиллагааг идэвхгүй болгосон.</translation>
+    </message>
     <message>
         <source>Change the passphrase used for wallet encryption</source>
         <translation type="unfinished">Түрүйвчийг цоожлох нууц үгийг солих</translation>
@@ -334,6 +343,46 @@
         <source>Error: %1</source>
         <translation type="unfinished">Алдаа: %1</translation>
     </message>
+    <message>
+        <source>Warning: %1</source>
+        <translation type="unfinished">Анхааруулга:%1</translation>
+    </message>
+    <message>
+        <source>Date: %1
+</source>
+        <translation type="unfinished">Огноо:%1
+</translation>
+    </message>
+    <message>
+        <source>Amount: %1
+</source>
+        <translation type="unfinished">Дүн: %1
+</translation>
+    </message>
+    <message>
+        <source>Wallet: %1
+</source>
+        <translation type="unfinished">Түрийвч: %1
+</translation>
+    </message>
+    <message>
+        <source>Type: %1
+</source>
+        <translation type="unfinished">Төрөл: %1
+</translation>
+    </message>
+    <message>
+        <source>Label: %1
+</source>
+        <translation type="unfinished">Шошго: %1
+</translation>
+    </message>
+    <message>
+        <source>Address: %1
+</source>
+        <translation type="unfinished">Хаяг: %1
+</translation>
+    </message>
     <message>
         <source>Sent transaction</source>
         <translation type="unfinished">Гадагшаа гүйлгээ</translation>
@@ -350,7 +399,11 @@
         <source>Wallet is &lt;b&gt;encrypted&lt;/b&gt; and currently &lt;b&gt;locked&lt;/b&gt;</source>
         <translation type="unfinished">Түрүйвч &lt;b&gt;цоожтой&lt;/b&gt; ба одоогоор цоож &lt;b&gt;хаалттай&lt;/b&gt; байна</translation>
     </message>
-    </context>
+    <message>
+        <source>Original message:</source>
+        <translation type="unfinished">Эх зурвас:</translation>
+    </message>
+</context>
 <context>
     <name>CoinControlDialog</name>
     <message>
@@ -438,6 +491,27 @@
         <source>Bitcoin</source>
         <translation type="unfinished">Биткойн</translation>
     </message>
+    <message numerus="yes">
+        <source>%n GB of space available</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>(of %n GB needed)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>(%n GB needed for full chain)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
     <message numerus="yes">
         <source>(sufficient to restore backups %n day(s) old)</source>
         <extracomment>Explanatory text on the capability of the current prune target.</extracomment>
@@ -500,6 +574,7 @@
     </message>
     <message>
         <source>Client restart required to activate changes.</source>
+        <extracomment>Text explaining that the settings changed will not come into effect until the client is restarted.</extracomment>
         <translation type="unfinished">Ѳѳрчлѳлтүүдийг идэвхижүүлхийн тулд клиентийг ахин эхлүүлэх шаардлагтай</translation>
     </message>
     <message>
@@ -779,10 +854,6 @@
         <source>Message:</source>
         <translation type="unfinished">Зурвас:</translation>
     </message>
-    <message>
-        <source>Pay To:</source>
-        <translation type="unfinished">Тѳлѳх хаяг:</translation>
-    </message>
     </context>
 <context>
     <name>SignVerifyMessageDialog</name>
@@ -799,10 +870,12 @@
     <name>TransactionDesc</name>
     <message>
         <source>%1/unconfirmed</source>
+        <extracomment>Text explaining the current status of a transaction, shown in the status field of the details window for this transaction. This status represents a transaction confirmed in at least one block, but less than 6 blocks.</extracomment>
         <translation type="unfinished">%1/баталгаажаагүй</translation>
     </message>
     <message>
         <source>%1 confirmations</source>
+        <extracomment>Text explaining the current status of a transaction, shown in the status field of the details window for this transaction. This status represents a transaction confirmed in 6 or more blocks.</extracomment>
         <translation type="unfinished">%1 баталгаажилтууд</translation>
     </message>
     <message>
@@ -989,6 +1062,10 @@
         <source>ID</source>
         <translation type="unfinished">Тодорхойлолт</translation>
     </message>
+    <message>
+        <source>Exporting Failed</source>
+        <translation type="unfinished">Экспорт амжилтгүй боллоо</translation>
+    </message>
     <message>
         <source>The transaction history was successfully saved to %1.</source>
         <translation type="unfinished">Гүйлгээнүй түүхийг %1-д амжилттай хадгаллаа.</translation>
@@ -1000,6 +1077,10 @@
 </context>
 <context>
     <name>WalletFrame</name>
+    <message>
+        <source>Create a new wallet</source>
+        <translation type="unfinished">Шинэ түрийвч үүсгэх</translation>
+    </message>
     <message>
         <source>Error</source>
         <translation type="unfinished">Алдаа</translation>
@@ -1023,4 +1104,15 @@
         <translation type="unfinished">Сонгогдсон таб дээрхи дата-г экспортлох</translation>
     </message>
     </context>
+<context>
+    <name>bitcoin-core</name>
+    <message>
+        <source>Done loading</source>
+        <translation type="unfinished">Ачааллаж дууслаа</translation>
+    </message>
+    <message>
+        <source>Insufficient funds</source>
+        <translation type="unfinished">Таны дансны үлдэгдэл хүрэлцэхгүй байна</translation>
+    </message>
+    </context>
 </TS>
\ No newline at end of file
