@@ -270,14 +270,6 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
         <extracomment>Explanatory text shown on startup when the settings file could not be written. Prompts user to check that we have the ability to write to the file. Explains that the user has the option of running without a settings file.</extracomment>
         <translation type="unfinished">Ýowuz ýalňyşlyk ýüze çykdy. Sazlamalar faýlyna ýazmak mümkinçiliginiň bardygyny ýa-da ýokdugyny barla, bolmasa -nosettings bilen işletmäge çalyş.</translation>
     </message>
-    <message>
-        <source>Error: Specified data directory "%1" does not exist.</source>
-        <translation type="unfinished">Ýalňyşlyk: Görkezilen maglumatlar katalogy "%1" ýok.</translation>
-    </message>
-    <message>
-        <source>Error: Cannot parse configuration file: %1.</source>
-        <translation type="unfinished">Ýalňyşlyk: %1 konfigurasiýa faýlyny derňäp bolanok.</translation>
-    </message>
     <message>
         <source>Error: %1</source>
         <translation type="unfinished">Ýalňyşlyk: %1</translation>
@@ -333,17 +325,6 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
         </translation>
     </message>
     </context>
-<context>
-    <name>bitcoin-core</name>
-    <message>
-        <source>Settings file could not be read</source>
-        <translation type="unfinished">Sazlamalar faýlyny okap bolanok</translation>
-    </message>
-    <message>
-        <source>Settings file could not be written</source>
-        <translation type="unfinished">Sazlamalar faýlyny ýazdyryp bolanok</translation>
-    </message>
-    </context>
 <context>
     <name>BitcoinGUI</name>
     <message>
@@ -519,10 +500,6 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
         <source>Processing blocks on disk…</source>
         <translation type="unfinished">Diskde bloklar işlenýär...</translation>
     </message>
-    <message>
-        <source>Reindexing blocks on disk…</source>
-        <translation type="unfinished">Diskde bloklar gaýtadan indekslenýär...</translation>
-    </message>
     <message>
         <source>Connecting to peers…</source>
         <translation type="unfinished">Deňdeşlere baglanylýar...</translation>
@@ -564,7 +541,7 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
     </message>
     <message>
         <source>Transactions after this will not yet be visible.</source>
-        <translation type="unfinished">Mundan soňky amallar entek görünmez.</translation>
+        <translation type="unfinished">Mundan soňky geleşikler entek görünmez.</translation>
     </message>
     <message>
         <source>Error</source>
@@ -650,6 +627,11 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
         <source>No wallets available</source>
         <translation type="unfinished">Elýeterli gapjyk ýok</translation>
     </message>
+    <message>
+        <source>Wallet Name</source>
+        <extracomment>Label of the input field where the name of the wallet is entered.</extracomment>
+        <translation type="unfinished">Gapjygyň ady</translation>
+    </message>
     <message>
         <source>&amp;Window</source>
         <translation type="unfinished">&amp;Penjire</translation>
@@ -679,7 +661,7 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
         <extracomment>A substring of the tooltip.</extracomment>
         <translation type="unfinished">
             <numerusform>Bitkoin toruna %n işjeň arabaglanyşyk.</numerusform>
-            <numerusform>Bitkoin toruna %n işjeň arabaglanyşyk.</numerusform>
+            <numerusform>Bitkoin ulgamyna %n işjeň arabaglanyşyk.</numerusform>
         </translation>
     </message>
     <message>
@@ -695,12 +677,12 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
     <message>
         <source>Disable network activity</source>
         <extracomment>A context menu item.</extracomment>
-        <translation type="unfinished">Toruň işjeňligini öçür</translation>
+        <translation type="unfinished">Ulgamyň işjeňligini öçür</translation>
     </message>
     <message>
         <source>Enable network activity</source>
         <extracomment>A context menu item. The network activity was disabled previously.</extracomment>
-        <translation type="unfinished">Toruň işjeňligini aç</translation>
+        <translation type="unfinished">Ulgamyň işjeňligini aç</translation>
     </message>
     <message>
         <source>Error: %1</source>
@@ -748,11 +730,11 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
     </message>
     <message>
         <source>Sent transaction</source>
-        <translation type="unfinished">Iberilen amal</translation>
+        <translation type="unfinished">Iberilen geleşik</translation>
     </message>
     <message>
         <source>Incoming transaction</source>
-        <translation type="unfinished">Gelýän amal</translation>
+        <translation type="unfinished">Gelýän geleşik</translation>
     </message>
     <message>
         <source>HD key generation is &lt;b&gt;enabled&lt;/b&gt;</source>
@@ -951,7 +933,7 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
     <message>
         <source>Creating Wallet &lt;b&gt;%1&lt;/b&gt;…</source>
         <extracomment>Descriptive text of the create wallet progress window which indicates to the user which wallet is currently being created.</extracomment>
-        <translation type="unfinished">Gapjyk döredilýär &lt;b&gt;%1&lt;/b&gt;...</translation>
+        <translation type="unfinished">&lt;b&gt;%1&lt;/b&gt; gapjyk döredilýär...</translation>
     </message>
     <message>
         <source>Create wallet failed</source>
@@ -965,7 +947,7 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
         <source>Can't list signers</source>
         <translation type="unfinished">Gol çekenleriň sanawyny görkezip bolanok</translation>
     </message>
-</context>
+    </context>
 <context>
     <name>OpenWalletActivity</name>
     <message>
@@ -983,7 +965,7 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
     <message>
         <source>Open Wallet</source>
         <extracomment>Title of window indicating the progress of opening of a wallet.</extracomment>
-        <translation type="unfinished">Gapjyk aç</translation>
+        <translation type="unfinished">Gapjygy aç</translation>
     </message>
     <message>
         <source>Opening Wallet &lt;b&gt;%1&lt;/b&gt;…</source>
@@ -1170,13 +1152,26 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
         <source>Bitcoin</source>
         <translation type="unfinished">Bitkoin</translation>
     </message>
-    <message>
-        <source>(of %1 GB needed)</source>
-        <translation type="unfinished">(Zerur bolan %1 GB-dan)</translation>
+    <message numerus="yes">
+        <source>%n GB of space available</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
-    <message>
-        <source>(%1 GB needed for full chain)</source>
-        <translation type="unfinished">(Doly zynjyr üçin %1 GB zerur)</translation>
+    <message numerus="yes">
+        <source>(of %n GB needed)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>(%n GB needed for full chain)</source>
+        <translation type="unfinished">
+            <numerusform />
+            <numerusform />
+        </translation>
     </message>
     <message>
         <source>At least %1 GB of data will be stored in this directory, and it will grow over time.</source>
@@ -1190,8 +1185,8 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
         <source>(sufficient to restore backups %n day(s) old)</source>
         <extracomment>Explanatory text on the capability of the current prune target.</extracomment>
         <translation type="unfinished">
-            <numerusform />
-            <numerusform />
+            <numerusform>(%n gün geçen ätiýaçlyk nusgalaryny dikeltmek üçin ýeterlik)</numerusform>
+            <numerusform>(%n gün geçen ätiýaçlyk nusgalaryny dikeltmek üçin ýeterlik)</numerusform>
         </translation>
     </message>
     <message>
@@ -1384,7 +1379,7 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
     <name>SendCoinsDialog</name>
     <message>
         <source>Quantity:</source>
-        <translation type="unfinished">Mukdar:</translation>
+        <translation type="unfinished">Sany:</translation>
     </message>
     <message>
         <source>Bytes:</source>
@@ -1396,7 +1391,7 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
     </message>
     <message>
         <source>Fee:</source>
-        <translation type="unfinished">Töleg:</translation>
+        <translation type="unfinished">Gatanç:</translation>
     </message>
     <message>
         <source>After Fee:</source>
@@ -1499,7 +1494,7 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
     </message>
     <message>
         <source>Copy transaction &amp;ID</source>
-        <translation type="unfinished">Amalyň &amp;ID-sini göçür</translation>
+        <translation type="unfinished">Geleşigiň &amp;ID-sini göçür</translation>
     </message>
     <message>
         <source>Comma separated file</source>
@@ -1556,4 +1551,15 @@ Diňe "miras" görnüşli salgylar bilen gol çekmek mümkin.</translation>
         <translation type="unfinished">Häzirki bellikdäki maglumaty faýla geçir</translation>
     </message>
     </context>
+<context>
+    <name>bitcoin-core</name>
+    <message>
+        <source>Settings file could not be read</source>
+        <translation type="unfinished">Sazlamalar faýlyny okap bolanok</translation>
+    </message>
+    <message>
+        <source>Settings file could not be written</source>
+        <translation type="unfinished">Sazlamalar faýlyny ýazdyryp bolanok</translation>
+    </message>
+</context>
 </TS>
\ No newline at end of file
