@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2021 The Bitcoin Core developers
+// Copyright (c) 2018-2022 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -22,7 +22,7 @@ static RPCHelpMan enumeratesigners()
         RPCResult{
             RPCResult::Type::OBJ, "", "",
             {
-                {RPCResult::Type::ARR, "signers", /* optional */ false, "",
+                {RPCResult::Type::ARR, "signers", /*optional=*/false, "",
                 {
                     {RPCResult::Type::OBJ, "", "",
                     {
@@ -62,15 +62,11 @@ static RPCHelpMan enumeratesigners()
     };
 }
 
-void RegisterSignerRPCCommands(CRPCTable &t)
+void RegisterSignerRPCCommands(CRPCTable& t)
 {
-// clang-format off
-static const CRPCCommand commands[] =
-{ // category              actor (function)
-  // --------------------- ------------------------
-  { "signer",              &enumeratesigners,      },
-};
-// clang-format on
+    static const CRPCCommand commands[]{
+        {"signer", &enumeratesigners},
+    };
     for (const auto& c : commands) {
         t.appendCommand(c.name, &c);
     }
