@@ -1,4 +1,4 @@
-// Copyright (c) 2012-2019 The Bitcoin Core developers
+// Copyright (c) 2012-2022 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -6,10 +6,10 @@
 #include <merkleblock.h>
 #include <serialize.h>
 #include <streams.h>
-#include <uint256.h>
-#include <arith_uint256.h>
-#include <version.h>
+#include <test/util/random.h>
 #include <test/util/setup_common.h>
+#include <uint256.h>
+#include <version.h>
 
 #include <vector>
 
@@ -70,7 +70,7 @@ BOOST_AUTO_TEST_CASE(pmt_test1)
             CPartialMerkleTree pmt1(vTxid, vMatch);
 
             // serialize
-            CDataStream ss(SER_NETWORK, PROTOCOL_VERSION);
+            DataStream ss{};
             ss << pmt1;
 
             // verify CPartialMerkleTree's size guarantees
@@ -107,13 +107,13 @@ BOOST_AUTO_TEST_CASE(pmt_test1)
 
 BOOST_AUTO_TEST_CASE(pmt_malleability)
 {
-    std::vector<uint256> vTxid = {
-        ArithToUint256(1), ArithToUint256(2),
-        ArithToUint256(3), ArithToUint256(4),
-        ArithToUint256(5), ArithToUint256(6),
-        ArithToUint256(7), ArithToUint256(8),
-        ArithToUint256(9), ArithToUint256(10),
-        ArithToUint256(9), ArithToUint256(10),
+    std::vector<uint256> vTxid{
+        uint256{1}, uint256{2},
+        uint256{3}, uint256{4},
+        uint256{5}, uint256{6},
+        uint256{7}, uint256{8},
+        uint256{9}, uint256{10},
+        uint256{9}, uint256{10},
     };
     std::vector<bool> vMatch = {false, false, false, false, false, false, false, false, false, true, true, false};
 
