@@ -1,27 +1,15 @@
 #!/usr/bin/env python3
-# Copyright (c) 2014-2021 The Bitcoin Core developers
+# Copyright (c) 2014-2022 The Bitcoin Core developers
 # Distributed under the MIT software license, see the accompanying
 # file COPYING or http://www.opensource.org/licenses/mit-license.php.
 """Test fee estimation code."""
+from copy import deepcopy
 from decimal import Decimal
 import os
 import random
 
 from test_framework.messages import (
     COIN,
-    COutPoint,
-    CTransaction,
-    CTxIn,
-    CTxOut,
-)
-from test_framework.script import (
-    CScript,
-    OP_1,
-    OP_DROP,
-    OP_TRUE,
-)
-from test_framework.script_util import (
-    script_to_p2sh_script,
 )
 from test_framework.test_framework import BitcoinTestFramework
 from test_framework.util import (
@@ -31,22 +19,14 @@ from test_framework.util import (
     assert_raises_rpc_error,
     satoshi_round,
 )
-
-# Construct 2 trivial P2SH's and the ScriptSigs that spend them
-# So we can create many transactions without needing to spend
-# time signing.
-SCRIPT = CScript([OP_1, OP_DROP])
-P2SH = script_to_p2sh_script(SCRIPT)
-REDEEM_SCRIPT = CScript([OP_TRUE, SCRIPT])
+from test_framework.wallet import MiniWallet
 
 
 def small_txpuzzle_randfee(
-    from_node, conflist, unconflist, amount, min_fee, fee_increment
+    wallet, from_node, conflist, unconflist, amount, min_fee, fee_increment, batch_reqs
 ):
-    """Create and send a transaction with a random fee.
+    """Create and send a transaction with a random fee using MiniWallet.
 
-    The transaction pays to a trivial P2SH script, and assumes that its inputs
-    are of the same form.
     The function takes a list of confirmed outputs and unconfirmed outputs
     and attempts to use the confirmed list first for its inputs.
     It adds the newly created outputs to the unconfirmed list.
@@ -58,25 +38,34 @@ def small_txpuzzle_randfee(
     rand_fee = float(fee_increment) * (1.1892 ** random.randint(0, 28))
     # Total fee ranges from min_fee to min_fee + 127*fee_increment
     fee = min_fee - fee_increment + satoshi_round(rand_fee)
-    tx = CTransaction()
+    utxos_to_spend = []
     total_in = Decimal("0.00000000")
     while total_in <= (amount + fee) and len(conflist) > 0:
         t = conflist.pop(0)
-        total_in += t["amount"]
-        tx.vin.append(CTxIn(COutPoint(int(t["txid"], 16), t["vout"]), REDEEM_SCRIPT))
+        total_in += t["value"]
+        utxos_to_spend.append(t)
     while total_in <= (amount + fee) and len(unconflist) > 0:
         t = unconflist.pop(0)
-        total_in += t["amount"]
-        tx.vin.append(CTxIn(COutPoint(int(t["txid"], 16), t["vout"]), REDEEM_SCRIPT))
+        total_in += t["value"]
+        utxos_to_spend.append(t)
     if total_in <= amount + fee:
         raise RuntimeError(f"Insufficient funds: need {amount + fee}, have {total_in}")
-    tx.vout.append(CTxOut(int((total_in - amount - fee) * COIN), P2SH))
-    tx.vout.append(CTxOut(int(amount * COIN), P2SH))
-    txid = from_node.sendrawtransaction(hexstring=tx.serialize().hex(), maxfeerate=0)
-    unconflist.append({"txid": txid, "vout": 0, "amount": total_in - amount - fee})
-    unconflist.append({"txid": txid, "vout": 1, "amount": amount})
+    tx = wallet.create_self_transfer_multi(
+        utxos_to_spend=utxos_to_spend,
+        fee_per_output=0,
+    )["tx"]
+    tx.vout[0].nValue = int((total_in - amount - fee) * COIN)
+    tx.vout.append(deepcopy(tx.vout[0]))
+    tx.vout[1].nValue = int(amount * COIN)
+    tx.rehash()
+    txid = tx.hash
+    tx_hex = tx.serialize().hex()
 
-    return (tx.serialize().hex(), fee)
+    batch_reqs.append(from_node.sendrawtransaction.get_request(hexstring=tx_hex, maxfeerate=0))
+    unconflist.append({"txid": txid, "vout": 0, "value": total_in - amount - fee})
+    unconflist.append({"txid": txid, "vout": 1, "value": amount})
+
+    return (tx.get_vsize(), fee)
 
 
 def check_raw_estimates(node, fees_seen):
@@ -129,17 +118,12 @@ def check_estimates(node, fees_seen):
     check_smart_estimates(node, fees_seen)
 
 
-def send_tx(node, utxo, feerate):
-    """Broadcast a 1in-1out transaction with a specific input and feerate (sat/vb)."""
-    tx = CTransaction()
-    tx.vin = [CTxIn(COutPoint(int(utxo["txid"], 16), utxo["vout"]), REDEEM_SCRIPT)]
-    tx.vout = [CTxOut(int(utxo["amount"] * COIN), P2SH)]
-
-    # vbytes == bytes as we are using legacy transactions
-    fee = tx.get_vsize() * feerate
-    tx.vout[0].nValue -= fee
-
-    return node.sendrawtransaction(tx.serialize().hex())
+def make_tx(wallet, utxo, feerate):
+    """Create a 1in-1out transaction with a specific input and feerate (sat/vb)."""
+    return wallet.create_self_transfer(
+        utxo_to_spend=utxo,
+        fee_rate=Decimal(feerate * 1000) / COIN,
+    )
 
 
 class EstimateFeeTest(BitcoinTestFramework):
@@ -152,9 +136,6 @@ class EstimateFeeTest(BitcoinTestFramework):
             ["-whitelist=noban@127.0.0.1", "-blockmaxweight=32000"],
         ]
 
-    def skip_test_if_missing_module(self):
-        self.skip_if_no_wallet()
-
     def setup_network(self):
         """
         We'll setup the network to have 3 nodes that all mine with different parameters.
@@ -168,9 +149,6 @@ class EstimateFeeTest(BitcoinTestFramework):
         # (68k weight is room enough for 120 or so transactions)
         # Node2 is a stingy miner, that
         # produces too small blocks (room for only 55 or so transactions)
-        self.start_nodes()
-        self.import_deterministic_coinbase_privkeys()
-        self.stop_nodes()
 
     def transact_and_mine(self, numblocks, mining_node):
         min_fee = Decimal("0.00001")
@@ -180,18 +158,23 @@ class EstimateFeeTest(BitcoinTestFramework):
         # resorting to tx's that depend on the mempool when those run out
         for _ in range(numblocks):
             random.shuffle(self.confutxo)
+            batch_sendtx_reqs = []
             for _ in range(random.randrange(100 - 50, 100 + 50)):
                 from_index = random.randint(1, 2)
-                (txhex, fee) = small_txpuzzle_randfee(
+                (tx_bytes, fee) = small_txpuzzle_randfee(
+                    self.wallet,
                     self.nodes[from_index],
                     self.confutxo,
                     self.memutxo,
                     Decimal("0.005"),
                     min_fee,
                     min_fee,
+                    batch_sendtx_reqs,
                 )
-                tx_kbytes = (len(txhex) // 2) / 1000.0
+                tx_kbytes = tx_bytes / 1000.0
                 self.fees_per_kb.append(float(fee) / tx_kbytes)
+            for node in self.nodes:
+                node.batch(batch_sendtx_reqs)
             self.sync_mempools(wait=0.1)
             mined = mining_node.getblock(self.generate(mining_node, 1)[0], True)["tx"]
             # update which txouts are confirmed
@@ -205,24 +188,10 @@ class EstimateFeeTest(BitcoinTestFramework):
 
     def initial_split(self, node):
         """Split two coinbase UTxOs into many small coins"""
-        utxo_count = 2048
-        self.confutxo = []
-        splitted_amount = Decimal("0.04")
-        fee = Decimal("0.1")
-        change = Decimal("100") - splitted_amount * utxo_count - fee
-        tx = CTransaction()
-        tx.vin = [
-            CTxIn(COutPoint(int(cb["txid"], 16), cb["vout"]))
-            for cb in node.listunspent()[:2]
-        ]
-        tx.vout = [CTxOut(int(splitted_amount * COIN), P2SH) for _ in range(utxo_count)]
-        tx.vout.append(CTxOut(int(change * COIN), P2SH))
-        txhex = node.signrawtransactionwithwallet(tx.serialize().hex())["hex"]
-        txid = node.sendrawtransaction(txhex)
-        self.confutxo = [
-            {"txid": txid, "vout": i, "amount": splitted_amount}
-            for i in range(utxo_count)
-        ]
+        self.confutxo = self.wallet.send_self_transfer_multi(
+            from_node=node,
+            utxos_to_spend=[self.wallet.get_utxo() for _ in range(2)],
+            num_outputs=2048)['new_utxos']
         while len(node.getrawmempool()) > 0:
             self.generate(node, 1, sync_fun=self.no_op)
 
@@ -282,14 +251,20 @@ class EstimateFeeTest(BitcoinTestFramework):
         assert_greater_than_or_equal(len(utxos), 250)
         for _ in range(5):
             # Broadcast 45 low fee transactions that will need to be RBF'd
+            txs = []
             for _ in range(45):
                 u = utxos.pop(0)
-                txid = send_tx(node, u, low_feerate)
+                tx = make_tx(self.wallet, u, low_feerate)
                 utxos_to_respend.append(u)
-                txids_to_replace.append(txid)
+                txids_to_replace.append(tx["txid"])
+                txs.append(tx)
             # Broadcast 5 low fee transaction which don't need to
             for _ in range(5):
-                send_tx(node, utxos.pop(0), low_feerate)
+                tx = make_tx(self.wallet, utxos.pop(0), low_feerate)
+                txs.append(tx)
+            batch_send_tx = [node.sendrawtransaction.get_request(tx["hex"]) for tx in txs]
+            for n in self.nodes:
+                n.batch(batch_send_tx)
             # Mine the transactions on another node
             self.sync_mempools(wait=0.1, nodes=[node, miner])
             for txid in txids_to_replace:
@@ -298,7 +273,12 @@ class EstimateFeeTest(BitcoinTestFramework):
             # RBF the low-fee transactions
             while len(utxos_to_respend) > 0:
                 u = utxos_to_respend.pop(0)
-                send_tx(node, u, high_feerate)
+                tx = make_tx(self.wallet, u, high_feerate)
+                node.sendrawtransaction(tx["hex"])
+                txs.append(tx)
+            dec_txs = [res["result"] for res in node.batch([node.decoderawtransaction.get_request(tx["hex"]) for tx in txs])]
+            self.wallet.scan_txs(dec_txs)
+
 
         # Mine the last replacement txs
         self.sync_mempools(wait=0.1, nodes=[node, miner])
@@ -316,6 +296,7 @@ class EstimateFeeTest(BitcoinTestFramework):
 
         # Split two coinbases into many small utxos
         self.start_node(0)
+        self.wallet = MiniWallet(self.nodes[0])
         self.initial_split(self.nodes[0])
         self.log.info("Finished splitting")
 
