@@ -1,29 +1,112 @@
 #!/usr/bin/env python3
-# Copyright (c) 2015-2021 The Bitcoin Core developers
+# Copyright (c) 2015-2022 The Bitcoin Core developers
 # Distributed under the MIT software license, see the accompanying
 # file COPYING or http://www.opensource.org/licenses/mit-license.php.
 """Test the prioritisetransaction mining RPC."""
 
+from decimal import Decimal
 import time
 
-from test_framework.messages import COIN, MAX_BLOCK_WEIGHT
+from test_framework.messages import (
+    COIN,
+    MAX_BLOCK_WEIGHT,
+)
 from test_framework.test_framework import BitcoinTestFramework
-from test_framework.util import assert_equal, assert_raises_rpc_error, create_confirmed_utxos, create_lots_of_big_transactions, gen_return_txouts
+from test_framework.util import (
+    assert_equal,
+    assert_raises_rpc_error,
+    create_lots_of_big_transactions,
+    gen_return_txouts,
+)
+from test_framework.wallet import MiniWallet
+
 
 class PrioritiseTransactionTest(BitcoinTestFramework):
     def set_test_params(self):
-        self.setup_clean_chain = True
         self.num_nodes = 1
         self.extra_args = [[
             "-printpriority=1",
-            "-acceptnonstdtxn=1",
+            "-datacarriersize=100000",
         ]] * self.num_nodes
         self.supports_cli = False
 
-    def skip_test_if_missing_module(self):
-        self.skip_if_no_wallet()
+    def test_diamond(self):
+        self.log.info("Test diamond-shape package with priority")
+        mock_time = int(time.time())
+        self.nodes[0].setmocktime(mock_time)
+
+        #      tx_a
+        #      / \
+        #     /   \
+        #   tx_b  tx_c
+        #     \   /
+        #      \ /
+        #      tx_d
+
+        tx_o_a = self.wallet.send_self_transfer_multi(
+            from_node=self.nodes[0],
+            num_outputs=2,
+        )
+        txid_a = tx_o_a["txid"]
+
+        tx_o_b, tx_o_c = [self.wallet.send_self_transfer(
+            from_node=self.nodes[0],
+            utxo_to_spend=u,
+        ) for u in tx_o_a["new_utxos"]]
+        txid_b = tx_o_b["txid"]
+        txid_c = tx_o_c["txid"]
+
+        tx_o_d = self.wallet.send_self_transfer_multi(
+            from_node=self.nodes[0],
+            utxos_to_spend=[
+                self.wallet.get_utxo(txid=txid_b),
+                self.wallet.get_utxo(txid=txid_c),
+            ],
+        )
+        txid_d = tx_o_d["txid"]
+
+        self.log.info("Test priority while txs are in mempool")
+        raw_before = self.nodes[0].getrawmempool(verbose=True)
+        fee_delta_b = Decimal(9999) / COIN
+        fee_delta_c_1 = Decimal(-1234) / COIN
+        fee_delta_c_2 = Decimal(8888) / COIN
+        self.nodes[0].prioritisetransaction(txid=txid_b, fee_delta=int(fee_delta_b * COIN))
+        self.nodes[0].prioritisetransaction(txid=txid_c, fee_delta=int(fee_delta_c_1 * COIN))
+        self.nodes[0].prioritisetransaction(txid=txid_c, fee_delta=int(fee_delta_c_2 * COIN))
+        raw_before[txid_a]["fees"]["descendant"] += fee_delta_b + fee_delta_c_1 + fee_delta_c_2
+        raw_before[txid_b]["fees"]["modified"] += fee_delta_b
+        raw_before[txid_b]["fees"]["ancestor"] += fee_delta_b
+        raw_before[txid_b]["fees"]["descendant"] += fee_delta_b
+        raw_before[txid_c]["fees"]["modified"] += fee_delta_c_1 + fee_delta_c_2
+        raw_before[txid_c]["fees"]["ancestor"] += fee_delta_c_1 + fee_delta_c_2
+        raw_before[txid_c]["fees"]["descendant"] += fee_delta_c_1 + fee_delta_c_2
+        raw_before[txid_d]["fees"]["ancestor"] += fee_delta_b + fee_delta_c_1 + fee_delta_c_2
+        raw_after = self.nodes[0].getrawmempool(verbose=True)
+        assert_equal(raw_before[txid_a], raw_after[txid_a])
+        assert_equal(raw_before, raw_after)
+
+        self.log.info("Test priority while txs are not in mempool")
+        self.restart_node(0, extra_args=["-nopersistmempool"])
+        self.nodes[0].setmocktime(mock_time)
+        assert_equal(self.nodes[0].getmempoolinfo()["size"], 0)
+        self.nodes[0].prioritisetransaction(txid=txid_b, fee_delta=int(fee_delta_b * COIN))
+        self.nodes[0].prioritisetransaction(txid=txid_c, fee_delta=int(fee_delta_c_1 * COIN))
+        self.nodes[0].prioritisetransaction(txid=txid_c, fee_delta=int(fee_delta_c_2 * COIN))
+        for t in [tx_o_a["hex"], tx_o_b["hex"], tx_o_c["hex"], tx_o_d["hex"]]:
+            self.nodes[0].sendrawtransaction(t)
+        raw_after = self.nodes[0].getrawmempool(verbose=True)
+        assert_equal(raw_before[txid_a], raw_after[txid_a])
+        assert_equal(raw_before, raw_after)
+
+        # Clear mempool
+        self.generate(self.nodes[0], 1)
+
+        # Use default extra_args
+        self.restart_node(0)
 
     def run_test(self):
+        self.wallet = MiniWallet(self.nodes[0])
+
         # Test `prioritisetransaction` required parameters
         assert_raises_rpc_error(-1, "prioritisetransaction", self.nodes[0].prioritisetransaction)
         assert_raises_rpc_error(-1, "prioritisetransaction", self.nodes[0].prioritisetransaction, '')
@@ -38,17 +121,22 @@ class PrioritiseTransactionTest(BitcoinTestFramework):
 
         # Test `prioritisetransaction` invalid `dummy`
         txid = '1d1d4e24ed99057e84c3f80fd8fbec79ed9e1acee37da269356ecea000000000'
-        assert_raises_rpc_error(-1, "JSON value is not a number as expected", self.nodes[0].prioritisetransaction, txid, 'foo', 0)
+        assert_raises_rpc_error(-3, "JSON value of type string is not of expected type number", self.nodes[0].prioritisetransaction, txid, 'foo', 0)
         assert_raises_rpc_error(-8, "Priority is no longer supported, dummy argument to prioritisetransaction must be 0.", self.nodes[0].prioritisetransaction, txid, 1, 0)
 
         # Test `prioritisetransaction` invalid `fee_delta`
-        assert_raises_rpc_error(-1, "JSON value is not an integer as expected", self.nodes[0].prioritisetransaction, txid=txid, fee_delta='foo')
+        assert_raises_rpc_error(-3, "JSON value of type string is not of expected type number", self.nodes[0].prioritisetransaction, txid=txid, fee_delta='foo')
+
+        self.test_diamond()
 
         self.txouts = gen_return_txouts()
         self.relayfee = self.nodes[0].getnetworkinfo()['relayfee']
 
         utxo_count = 90
-        utxos = create_confirmed_utxos(self, self.relayfee, self.nodes[0], utxo_count)
+        utxos = self.wallet.send_self_transfer_multi(from_node=self.nodes[0], num_outputs=utxo_count)['new_utxos']
+        self.generate(self.wallet, 1)
+        assert_equal(len(self.nodes[0].getrawmempool()), 0)
+
         base_fee = self.relayfee*100 # our transactions are smaller than 100kb
         txids = []
 
@@ -58,7 +146,13 @@ class PrioritiseTransactionTest(BitcoinTestFramework):
             txids.append([])
             start_range = i * range_size
             end_range = start_range + range_size
-            txids[i] = create_lots_of_big_transactions(self.nodes[0], self.txouts, utxos[start_range:end_range], end_range - start_range, (i+1)*base_fee)
+            txids[i] = create_lots_of_big_transactions(
+                self.wallet,
+                self.nodes[0],
+                (i+1) * base_fee,
+                end_range - start_range,
+                self.txouts,
+                utxos[start_range:end_range])
 
         # Make sure that the size of each group of transactions exceeds
         # MAX_BLOCK_WEIGHT // 4 -- otherwise the test needs to be revised to
@@ -117,17 +211,9 @@ class PrioritiseTransactionTest(BitcoinTestFramework):
                 assert x not in mempool
 
         # Create a free transaction.  Should be rejected.
-        utxo_list = self.nodes[0].listunspent()
-        assert len(utxo_list) > 0
-        utxo = utxo_list[0]
-
-        inputs = []
-        outputs = {}
-        inputs.append({"txid" : utxo["txid"], "vout" : utxo["vout"]})
-        outputs[self.nodes[0].getnewaddress()] = utxo["amount"]
-        raw_tx = self.nodes[0].createrawtransaction(inputs, outputs)
-        tx_hex = self.nodes[0].signrawtransactionwithwallet(raw_tx)["hex"]
-        tx_id = self.nodes[0].decoderawtransaction(tx_hex)["txid"]
+        tx_res = self.wallet.create_self_transfer(fee_rate=0)
+        tx_hex = tx_res['hex']
+        tx_id = tx_res['txid']
 
         # This will raise an exception due to min relay fee not being met
         assert_raises_rpc_error(-26, "min relay fee not met", self.nodes[0].sendrawtransaction, tx_hex)
